/* utilSlist.c: Functions for singly linked lists
 */

#include "utilSlist.h"
#include <malloc.h>

/* --- Functions that apply to singly linked lists in general --- */

static void *(*theMallocFunc)(size_t) = malloc;
static void (*theFreeFunc)(void *) = free;


/* --- Functions that apply to headerless lists --- */

void useFuncsForSlists(
  void *(*mallocFunc)(size_t),
  void (*freeFunc)(void *)
)
{
  theMallocFunc = mallocFunc;
  theFreeFunc = freeFunc;
}

slist *createSlist()
{
  return NULL;
}

void destroySlist(slist *theList)
{
  slist *this;
  slist *next;

  this = theList;

  while(this != NULL)
  {
    next = this->next;
    theFreeFunc(this);
    this = next;
  }
}

slist *addToSlistStart(slist *theList, itemType *data)
{
  slist *newNode;

  newNode = (slist *)theMallocFunc(sizeof(slist));
  newNode->data = data;
  newNode->next = theList;

  return newNode;
}

int doAllSlist(
  slist *theList,
  int (*itemFunc)(itemType *, char *),
  char *data
)
{
  slist *this;
  int result;

  this = theList;

  while(this != NULL)
  {
    result = itemFunc(this->data, data);
    if(result != 0) return result;
    this = this->next;
  }
  return 0; /* success */
}

int sizeOfSlist(slist *theList)
{
  int result = 0;
  slist *this;
  this = theList;

  while(this !=NULL)
  {
    result++;
    this = this->next;
  }
  return result;
}



/* --- Functions that apply to lists with a header & end ptr --- */

hslist *createHslist()
{
  hslist *newList;
  newList = (hslist *)theMallocFunc(sizeof(hslist));
  newList->size = 0;
  newList->first = NULL;
  newList->last = NULL;

  return newList;
}

void destroyHslist(hslist *theList)
{
  destroySlistPointer(theList->first);
  theFreeFunc(theList);
}

hslist *addToHslistStart(hslist *theList, itemType *data)
{
  slist *newNode;
  hslist *actualList;

  if(theList == NULL) actualList = createHslist();
  else actualList = theList;

  newNode = addToSlistStart(actualList->first, data);
  if(actualList->first == NULL)
  {
    actualList->last = newNode;
  }
  actualList->first = newNode;
  actualList->size++;

  return actualList;
}
